'use strict';

var _defineProperty = require('@babel/runtime/helpers/defineProperty');
var _slicedToArray = require('@babel/runtime/helpers/slicedToArray');
var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');
var _assertThisInitialized = require('@babel/runtime/helpers/assertThisInitialized');
var _inherits = require('@babel/runtime/helpers/inherits');
var _possibleConstructorReturn = require('@babel/runtime/helpers/possibleConstructorReturn');
var _getPrototypeOf = require('@babel/runtime/helpers/getPrototypeOf');
var _wrapNativeSuper = require('@babel/runtime/helpers/wrapNativeSuper');
var _typeof = require('@babel/runtime/helpers/typeof');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var _defineProperty__default = /*#__PURE__*/_interopDefaultLegacy(_defineProperty);
var _slicedToArray__default = /*#__PURE__*/_interopDefaultLegacy(_slicedToArray);
var _classCallCheck__default = /*#__PURE__*/_interopDefaultLegacy(_classCallCheck);
var _assertThisInitialized__default = /*#__PURE__*/_interopDefaultLegacy(_assertThisInitialized);
var _inherits__default = /*#__PURE__*/_interopDefaultLegacy(_inherits);
var _possibleConstructorReturn__default = /*#__PURE__*/_interopDefaultLegacy(_possibleConstructorReturn);
var _getPrototypeOf__default = /*#__PURE__*/_interopDefaultLegacy(_getPrototypeOf);
var _wrapNativeSuper__default = /*#__PURE__*/_interopDefaultLegacy(_wrapNativeSuper);
var _typeof__default = /*#__PURE__*/_interopDefaultLegacy(_typeof);

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty__default['default'](target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf__default['default'](Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf__default['default'](this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn__default['default'](this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

/**
 * Config options
 * @typedef {Object.<string, *>} Config
 * @property {string} [method = "GET"] - HTTP request method
 * @property {string} [url] - The url
 * @property {Object.<string, string>} [headers = {}] - The request headers
 * @property {Object.<string, *>} [params] - The request query
 * @property {*} [data = null] - The data to be sent
 * @property {number} [maxRedirects = 5] - Maximum redirects before error is thrown
 * @property {boolean} [withCredentials = false] - Whether cross-site Access-Control requests should be made using credentials
 *
 * Regrest response object
 * @typedef {Object.<string, *>} Response
 * @property {number} status - The response status code
 * @property {string} statusText - The response status text
 * @property {Object.<string, string>} headers - The response headers
 * @property {string} text - The response raw text
 * @property {Object} json - The response parsed as JSON
 * @property {Blob|any} arrayBuffer - The response as an Blob on browser or Buffer on Node js
 * @property {Blob} blob - The response as a Blob object
 */
var ENVIRONMENTS = Object.freeze({
  BROWSER: 0,
  NODE: 1,
  UNKNOWN: 2
}); // Detect whether instance is ran in browser or on node js

var ENV = (typeof window === "undefined" ? "undefined" : _typeof__default['default'](window)) === "object" && _typeof__default['default'](window.document) === "object" ? ENVIRONMENTS.BROWSER : (typeof module === "undefined" ? "undefined" : _typeof__default['default'](module)) === "object" && module && _typeof__default['default'](module.exports) === "object" ? ENVIRONMENTS.NODE : ENVIRONMENTS.UNKNOWN;

var NetworkError = /*#__PURE__*/function (_Error) {
  _inherits__default['default'](NetworkError, _Error);

  var _super = _createSuper(NetworkError);

  function NetworkError(message, request, response) {
    var _this;

    _classCallCheck__default['default'](this, NetworkError);

    _this = _super.call(this, message);
    _this.name = _this.constructor.name;
    _this.response = response;
    _this.request = request;

    if (typeof Error.captureStackTrace === "function") {
      Error.captureStackTrace(_assertThisInitialized__default['default'](_this), _this.constructor);
    } else {
      _this.stack = new Error(message).stack;
    }

    return _this;
  }

  return NetworkError;
}( /*#__PURE__*/_wrapNativeSuper__default['default'](Error));

function Regrest() {
  switch (ENV) {
    case ENVIRONMENTS.BROWSER:
      this.requestAdapter = xhrAdapter.bind(this);
      break;

    case ENVIRONMENTS.NODE:
      this.nodeAdapters = {
        http: require("follow-redirects").http,
        https: require("follow-redirects").https
      };
      this.requestAdapter = httpAdapter.bind(this);
      break;

    default:
      throw new Error("Unsupported environment");
  }
}
/**
 * @param {Config} config
 * @param {string} config.url - The url
 * @returns {Promise<Response>}
 * @memberof Regrest
 */


Regrest.prototype.request = function (_ref) {
  var _ref$method = _ref.method,
      method = _ref$method === void 0 ? "GET" : _ref$method,
      url = _ref.url,
      _ref$headers = _ref.headers,
      headers = _ref$headers === void 0 ? {} : _ref$headers,
      params = _ref.params,
      _ref$data = _ref.data,
      data = _ref$data === void 0 ? null : _ref$data,
      _ref$maxRedirects = _ref.maxRedirects,
      maxRedirects = _ref$maxRedirects === void 0 ? 5 : _ref$maxRedirects,
      _ref$withCredentials = _ref.withCredentials,
      withCredentials = _ref$withCredentials === void 0 ? false : _ref$withCredentials;
  // Generate query string and join it with url
  url = "".concat(url).concat(params ? "?".concat(Object.entries(params).map(function (_ref2) {
    var _ref3 = _slicedToArray__default['default'](_ref2, 2),
        key = _ref3[0],
        value = _ref3[1];

    return "".concat(key, "=").concat(value);
  }).join("&")) : "");
  return this.requestAdapter(method, url, data, headers, maxRedirects, withCredentials);
}; // Convenience methods

/**
 * @param {string} url - The url
 * @param {Config} [config] - Config
 * @returns {Promise<Response>}
 * @memberof Regrest
 */


["get", "head", "delete", "options"].forEach(function (method) {
  return Regrest.prototype[method] = function (url, config) {
    return this.request(_objectSpread(_objectSpread({}, config), {}, {
      method: method.toUpperCase(),
      url: url
    }));
  };
});
/**
 * @param {string} url - The url
 * @param {*} [data] - The data to be sent
 * @param {Config} [config] - Config
 * @returns {Promise<Response>}
 * @memberof Regrest
 */

["post", "put", "patch"].forEach(function (method) {
  return Regrest.prototype[method] = function (url, data, config) {
    return this.request(_objectSpread(_objectSpread({}, config), {}, {
      method: method.toUpperCase(),
      url: url,
      data: data
    }));
  };
}); // Unexposed helper methods and adapters

function xhrAdapter(requestType, url, body, headers, _, withCredentials) {
  return new Promise(function (resolve, reject) {
    var request = new XMLHttpRequest();
    request.open(requestType, url, true);
    Object.entries(headers).forEach(function (_ref4) {
      var _ref5 = _slicedToArray__default['default'](_ref4, 2),
          key = _ref5[0],
          value = _ref5[1];

      return request.setRequestHeader(key, value);
    });
    withCredentials && (request.withCredentials = true);

    request.onload = function () {
      var contentType = (this.getResponseHeader("Content-Type") || "").split(";")[0].trim();
      var response = {
        status: this.status,
        statusText: this.statusText,
        headers: _objectSpread({}, this.getAllResponseHeaders().trim().split(/[\r\n]+/).map(function (header) {
          return header.split(": ");
        }).reduce(function (obj, _ref6) {
          var _ref7 = _slicedToArray__default['default'](_ref6, 2),
              key = _ref7[0],
              value = _ref7[1];

          return _objectSpread(_objectSpread({}, obj), {}, _defineProperty__default['default']({}, key.toLowerCase(), value));
        }, {})),
        text: this.responseText,

        get json() {
          var result = JSON.parse(this.text);
          delete this.json;
          return this.json = result;
        },

        get blob() {
          var result = new Blob([request.response], {
            type: contentType
          });
          delete this.blob;
          return this.blob = result;
        },

        get arrayBuffer() {
          delete this.arrayBuffer;
          return this.arrayBuffer = this.blob;
        }

      };

      if (this.status >= 200 && this.status < 400) {
        resolve(response);
      } else {
        reject(new NetworkError("".concat(this.status, " ").concat(this.statusText), request, response));
      }
    };

    request.onerror = function () {
      reject(new NetworkError("connection error", request));
    };

    request.send(body);
  });
}

function httpAdapter(requestType, url, body, headers, maxRedirects) {
  var _this2 = this;

  return new Promise(function (resolve, reject) {
    var parsedUrl = new URL(url);
    var options = {
      host: parsedUrl.host,
      path: "".concat(parsedUrl.pathname).concat(parsedUrl.search),
      method: requestType,
      headers: headers,
      maxRedirects: maxRedirects
    };

    var req = _this2.nodeAdapters[parsedUrl.protocol.slice(0, -1)].request(options, function (res) {
      var response = {
        status: res.statusCode,
        statusText: res.statusMessage,
        headers: res.headers,
        arrayBuffer: [],

        get text() {
          var result = this.arrayBuffer.toString("utf-8");
          delete this.text;
          return this.text = result;
        },

        get json() {
          var result = JSON.parse(this.text);
          delete this.json;
          return this.json = result;
        },

        get blob() {
          if (Blob === undefined) {
            throw new Error("Please include Blob polyfill for Node.js");
          }

          var contentType = this.headers["content-type"] || "";
          var result = new Blob([new Uint8Array(this.arrayBuffer)], {
            type: contentType.split(";")[0].trim()
          });
          delete this.blob;
          return this.blob = result;
        }

      };
      res.on("data", function (chunk) {
        return response.arrayBuffer.push(chunk);
      });
      res.on("end", function () {
        response.arrayBuffer = Buffer.concat(response.arrayBuffer);

        if (res.statusCode >= 200 && res.statusCode < 400) {
          resolve(response);
        } else {
          reject(new NetworkError("".concat(res.statusCode, " ").concat(res.statusMessage), req, response));
        }
      });
    });

    req.on("error", function (e) {
      return reject(new NetworkError(e.message, req));
    });
    body && req.write(body);
    req.end();
  });
}

var index = new Regrest();

module.exports = index;
